/*
 ## Cypress USB 3.0 Platform source file (cyfxslfifosync.c)
 ## ===========================
 ##
 ##  Copyright Cypress Semiconductor Corporation, 2010-2011,
 ##  All Rights Reserved
 ##  UNPUBLISHED, LICENSED SOFTWARE.
 ##
 ##  CONFIDENTIAL AND PROPRIETARY INFORMATION
 ##  WHICH IS THE PROPERTY OF CYPRESS.
 ##
 ##  Use of this file is governed
 ##  by the license agreement included in the file
 ##
 ##     <install>/license/license.txt
 ##
 ##  where <install> is the Cypress software
 ##  installation root directory path.
 ##
 ## ===========================
*/

/* This file illustrates the Slave FIFO Synchronous mode example */

/*
   This example comprises of two USB bulk endpoints. A bulk OUT endpoint acts as the
   producer of data from the host. A bulk IN endpoint acts as the consumer of data to
   the host. Appropriate vendor class USB enumeration descriptors with these two bulk
   endpoints are implemented.

   The GPIF configuration data for the Synchronous Slave FIFO operation is loaded onto
   the appropriate GPIF registers. The p-port data transfers are done via the producer
   p-port socket and the consumer p-port socket.

   This example implements two DMA Channels in MANUAL mode one for P to U data transfer
   and one for U to P data transfer.

   The U to P DMA channel connects the USB producer (OUT) endpoint to the consumer p-port
   socket. And the P to U DMA channel connects the producer p-port socket to the USB 
   consumer (IN) endpoint.

   Upon every reception of data in the DMA buffer from the host or from the p-port, the
   CPU is signalled using DMA callbacks. There are two DMA callback functions implemented
   each for U to P and P to U data paths. The CPU then commits the DMA buffer received so
   that the data is transferred to the consumer.

   The DMA buffer size for each channel is defined based on the USB speed. 64 for full
   speed, 512 for high speed and 1024 for super speed. CY_FX_SLFIFO_DMA_BUF_COUNT in the
   header file defines the number of DMA buffers per channel.

   The constant CY_FX_SLFIFO_GPIF_16_32BIT_CONF_SELECT in the header file is used to
   select 16bit or 32bit GPIF data bus configuration.
 */

#include "cyu3system.h"
#include "cyu3os.h"
#include "cyu3dma.h"
#include "cyu3error.h"
#include "cyu3usb.h"
#include "cyu3uart.h"
#include "cyfxslfifosync.h"
#include "cyu3gpif.h"
#include "cyu3pib.h"
#include "pib_regs.h"
#include "cyu3i2c.h"
#include "cyu3sib.h" //data types and APIs that support SD/MMC/SDIO

#include "cyu3gpio.h"
#include "cyu3utils.h"
#include "cyfxusbi2cregmode.h"
#include "gpio_regs.h"
/* This file should be included only once as it contains
 * structure definitions. Including it in multiple places
 * can result in linker error. */
#include "cyfxgpif_syncsf.h"
#include "gpif.h"
#define COMMAND_BYTES 1000
#define Send_command_bytes 3
#define Recv_command_bytes 8

typedef unsigned char  uint8_t;
typedef unsigned short uint16_t;
typedef unsigned long  uint32_t;

uint8_t rxcmplt=0;

uint16_t glSpiPageSize = 0x100;  /* SPI Page size to be used for transfers. */
uint8_t glEp0Buffer[COMMAND_BYTES] __attribute__ ((aligned (32)));

CyU3PThread appThread;
CyU3PThread slFifoAppThread;	        /* Slave FIFO application thread structure */
CyU3PDmaChannel glChHandleSlFifoUtoP;   /* DMA Channel handle for U2P transfer. */
CyU3PDmaChannel glChHandleSlFifoPtoU;   /* DMA Channel handle for P2U transfer. */
CyU3PDmaChannel glChHandleSlFifoPtoS;
CyU3PDmaChannel glChHandleBulkSrc;       /* DMA MANUAL_OUT channel handle.*/

uint32_t glDMARxCount = 0;               /* Counter to track the number of buffers received from USB. */
uint32_t glDMATxCount = 0;               /* Counter to track the number of buffers sent to USB. */
CyBool_t glIsApplnActive = CyFalse;      /* Whether the loopback application is active or not. */
uint8_t flag=0; uint8_t flagy=0;

CyBool_t glDataTransStarted = CyFalse;   /* Whether DMA transfer has been started after enumeration. */
//void abcd();
 void xyz();
 //void gpif_error_cb(CyU3PPibIntrType cbType, uint16_t cbArg);
uint8_t location[COMMAND_BYTES];     int i;
uint8_t location1[50];
uint8_t READBUFFER[COMMAND_BYTES];
uint8_t *decodebuffer;
int byte_count=0;
int x=0;
uint8_t receivecom[50];
int ZYNQ_COMM=0;
int FX3_COMM=0;
int get_flag=0;
int sum=0;
uint8_t pulse_flag=0;
uint8_t recieved_from_usb=0;
//uint8_t flagy=0;
uint8_t receivecom_flag=0;
int checksunm_flag=0;
int checksunm_flag1=0;
//void xyz();
short ASCI_HEX(short O)
{
	if(O<=0x39)
	{
		O=O-0x30;
	}
	else
	{
		O=O-0x37;
	}
	return(O);
}

char HEX_ASCI(volatile char V)
{
	if(V<0x0a)
	{
		V=V+0x30;
	}
	else
	{
		V=V+0x37;
	}
	return (V);
}


int intoarr(int number,unsigned char arrayn[])
	    	{
	    		int c=0,n = number;
	    		while(number!=0)
	    		{
	    			number /= 10;
	    			c++;
	    		}

c=c-1;
	    			while (c != -1)
	    			{
	    				if(n==0)
	    				{
	    					arrayn[c]=0;
	    				}
	    				else
	    				{
	    					arrayn[c] = n % 10;
	    					n /= 10;
	    				}

	    				c--;
	    			}

	    		return 0;
	    	}
/* Application Error Handler */
void
CyFxAppErrorHandler (
        CyU3PReturnStatus_t apiRetStatus    /* API return status */
        )
{
    /* Application failed with the error code apiRetStatus */

    /* Add custom debug or recovery actions here */

    /* Loop Indefinitely */
    for (;;)
    {
        /* Thread sleep : 100 ms */
        CyU3PThreadSleep (100);
    }
}

/* This function initializes the debug module. The debug prints
 * are routed to the UART and can be seen using a UART console
 * running at 115200 baud rate. */
void
CyFxSlFifoApplnDebugInit (void)
{
    CyU3PUartConfig_t uartConfig;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Initialize the UART for printing debug messages */
    apiRetStatus = CyU3PUartInit();
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        /* Error handling */
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Set UART configuration */
    CyU3PMemSet ((uint8_t *)&uartConfig, 0, sizeof (uartConfig));
    uartConfig.baudRate = CY_U3P_UART_BAUDRATE_115200;
    uartConfig.stopBit = CY_U3P_UART_ONE_STOP_BIT;
    uartConfig.parity = CY_U3P_UART_NO_PARITY;
    uartConfig.txEnable = CyTrue;
    uartConfig.rxEnable = CyFalse;
    uartConfig.flowCtrl = CyFalse;
    uartConfig.isDma = CyTrue;

    apiRetStatus = CyU3PUartSetConfig (&uartConfig, 0);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Set the UART transfer to a really large value. */
    apiRetStatus = CyU3PUartTxSetBlockXfer (0xFFFFFFFF);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Initialize the debug module. */
    apiRetStatus = CyU3PDebugInit (CY_U3P_LPP_SOCKET_UART_CONS, 8);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyFxAppErrorHandler(apiRetStatus);
    }
}




/* DMA callback function to handle the produce events for P to U transfers. */
//void
//CyFxSlFifoPtoUDmaCallback (
//        CyU3PDmaChannel   *chHandle,
//        CyU3PDmaCbType_t  type,
//        CyU3PDmaCBInput_t *input
//        )
//{
//    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
//
//    if (type == CY_U3P_DMA_CB_PROD_EVENT)
//    {
//        /* This is a produce event notification to the CPU. This notification is
//         * received upon reception of every buffer. The buffer will not be sent
//         * out unless it is explicitly committed. The call shall fail if there
//         * is a bus reset / usb disconnect or if there is any application error. */
//        status = CyU3PDmaChannelCommitBuffer (chHandle, input->buffer_p.count, 0);
//        if (status != CY_U3P_SUCCESS)
//                   {
//                       CyU3PDebugPrint (4, "CyU3PDmaChannelCommitBuffer failed, Error code = %d\n", status);
//                   }
//        /* Increment the counter. */
//        glDMATxCount++;
//    }
//}
//

void
CyFxSlFifoPtoSDmaCallback(
        CyU3PDmaChannel   *chHandle,
        CyU3PDmaCbType_t  type,
        CyU3PDmaCBInput_t *input
        )
{
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

    if (type == CY_U3P_DMA_CB_PROD_EVENT)
    {
        status = CyU3PDmaChannelCommitBuffer (chHandle, input->buffer_p.count, 0);

        /* Increment the counter. */
        glDMARxCount++;
    }

    /* here i am writing to sdcard socket 0 */
    /* is it right way? */
    /* i am using p port producer socket 1 and s port consumer socket 0*/
    if (type == CY_U3P_DMA_CB_RECV_CPLT)
        {
    	if (status != CY_U3P_SUCCESS)
    	            {

    	        	status = CyU3PSibReadWriteRequest (0, 0,
    	        			4, 2, 0, 0);
    	            }

        }

    if (type == CY_U3P_DMA_CB_CONS_EVENT )
    {
    	rxcmplt++;
    }
}

/* Callback funtion for the DMA event notification. */
void
CyFxBulkSrcSinkDmaCallback (
        CyU3PDmaChannel   *chHandle, /* Handle to the DMA channel. */
        CyU3PDmaCbType_t  type,      /* Callback type.             */
        CyU3PDmaCBInput_t *input)    /* Callback status.           */
{
    CyU3PDmaBuffer_t buf_p;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;

    glDataTransStarted = CyTrue;
    if (type == CY_U3P_DMA_CB_CONS_EVENT ) //CY_U3P_DMA_CB_CONS_EVENT
    {
        //    	 for(i=0;i<=COMMAND_BYTES;i++)
//    	       	 {
//    	        buf_p.buffer[i]=READBUFFER[i];
//    	       	 }
         status = CyU3PDmaChannelGetBuffer (chHandle, &buf_p, CYU3P_NO_WAIT);
        if (status == CY_U3P_SUCCESS)
        {
            /* Commit the full buffer with default status. */
            status = CyU3PDmaChannelCommitBuffer (chHandle,COMMAND_BYTES , 0); //
            if (status != CY_U3P_SUCCESS)
            {
                CyU3PDebugPrint (4, "CyU3PDmaChannelCommitBuffer failed, Error code = %d\n", status);
            }
        }
        else
        {
            CyU3PDebugPrint (4, "CyU3PDmaChannelGetBuffer failed, Error code = %d\n", status);
        }

        /* Increment the counter. */
        glDMATxCount++;
   }
}
// added by geethanjali 10-04-2019//
void CyFxBulkSrcSinkDmaCallback1(CyU3PDmaChannel   *chHandle,
        CyU3PDmaCbType_t  type,
        CyU3PDmaCBInput_t *input)
{
	//CyU3PDmaBuffer_t buf_p;
	    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
	    unsigned char checksum[3];
	    glDataTransStarted = CyTrue;

	   // CyU3PUsbLPMDisable();
	    if (type == CY_U3P_DMA_CB_PROD_EVENT)
	    {
	       status = CyU3PDmaChannelCommitBuffer (chHandle, input->buffer_p.count, 0);
	       if (status != CY_U3P_SUCCESS)
	                  {
	                      CyU3PDebugPrint (4, "CyU3PDmaChannelCommitBuffer failed, Error code = %d\n", status);
	                  }
	        {
				{
				 flag=0x55;
				 byte_count=input->buffer_p.count;

				 for(i=0;i<=byte_count;i++)
				  {
					location[x++]= input->buffer_p.buffer[i];
					location1[i]= input->buffer_p.buffer[i];

					if(x>=COMMAND_BYTES)
														{
															x=0;
														}
					  }

						//memcopy operation(continuous command length storage)
											 receivecom_flag=1;

										 /* checksum calculation for receiving commands from gui side*/

											 				 int i=0 , sum,j=0,a,b,x1;
											 				 if(location1[i]=='$')
											 				 				 	{
											 				 				 		sum=0;
											 				 				 		j=i+1;
											 				 				 		while(location1[i]!='#')
											 				 				 		{

											 				 				 			sum=sum+location1[i];
											 				 				 			i++;
											 				 				 		}
											 				 				 		sum=sum+location1[i];
											 				 				 		intoarr(sum,checksum);
											 				 				 		a=ASCI_HEX(location1[i+1]);
											 				 				 		b=ASCI_HEX(location1[i+2]);
											 				 				 		if(a==checksum[0] && b ==checksum[1])
											 				 				 		{
											 				 				 			checksunm_flag=1;
											 				 				 			//return CyTrue;
											 				 				 		}
											 				}

											 				 /*finding fx3 commands or zynq commands*/
											 				 x1=location1[j];
											 								 				 	switch(x1)
											 								 				 	{
											 								 				 	case 'Z':
											 								 				 		ZYNQ_COMM=1;
											 								 				 	 memcpy(receivecom, location1, sizeof(location1));
											 								 				 	break;
											 								 				 	case 'F':
											 								 				 		FX3_COMM=1;
											 								 				 	break;
											 								 				 	default:
											 								 				 	break;
											 								 				 	}

											 					        }


												  }
								    recieved_from_usb=1;
										//	 decodebuffer=location;
			//	 decodebuffer=location;


	        status = CyU3PDmaChannelDiscardBuffer (chHandle);
	        //Increment the counter.
	        glDMARxCount++;
	    }
}
/*
 * Fill all DMA buffers on the IN endpoint with data. This gets data moving after an endpoint reset.
 */
static void
CyFxBulkSrcSinkFillInBuffers (
        void)
{
	CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PDmaBuffer_t    buf_p;
    uint16_t            index = 0;
   // CyU3PReturnStatus_t stat;

    /* Now preload all buffers in the MANUAL_OUT pipe with the required data. */
    for (index = 0; index <100; index++)
    {
    	status = CyU3PDmaChannelGetBuffer (&glChHandleBulkSrc, &buf_p, CYU3P_NO_WAIT);
    	 if (status != CY_U3P_SUCCESS)
    	            {
    	                CyU3PDebugPrint (4, "CyU3PDmaChannelCommitBuffer failed, Error code = %d\n", status);
    	            }
        	for(i=0;i<=COMMAND_BYTES;i++)
        	{
        		buf_p.buffer[i]=READBUFFER[i];
        	}

      //  CyU3PMemSet (buf_p.buffer, CY_FX_BULKSRCSINK_PATTERN, buf_p.size);
        	status = CyU3PDmaChannelCommitBuffer (&glChHandleBulkSrc,COMMAND_BYTES, 0);
    }
}

CyU3PReturnStatus_t
CyFxUsbI2cTransfer (
		uint16_t  byteCount,
		 uint8_t  *buffer,
        CyBool_t  isRead)
{
    CyU3PI2cPreamble_t preamble;
   // uint16_t pageCount = (byteCount / glI2cPageSize);
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
   // uint8_t  *buffer;

  //  uint16_t resCount = glI2cPageSize;

 /*   if (byteCount == 0)
    {
        return CY_U3P_SUCCESS;
    }*/

  /* if ((byteCount % glI2cPageSize) != 0)
    {
        pageCount ++;
        resCount = byteCount % glI2cPageSize;
    }*/



   // while (pageCount != 0)
   // {
        if (isRead)
        {
            /* Update the preamble information. */
            preamble.length    = 3;
            preamble.buffer[0] = 0x02; // u3z2 0x02;
            preamble.buffer[1] = 0x45;
            preamble.buffer[2] = 0x03;// u3z2 0X03;
            preamble.ctrlMask  = 0x0004;

            status = CyU3PI2cReceiveBytes (&preamble, buffer,byteCount, 1);
            if (status != CY_U3P_SUCCESS)
            {
                return status;
            }
        }
        else /* Write */
        {
            /* Update the preamble information. */
            preamble.length    = 1;
            preamble.buffer[0] = 0x02;// u3z2 0x02;
            preamble.ctrlMask  = 0x0000;

//while(1)buffer[0]='$';
          /*  buffer[1]='Z';
            buffer[2]='Y';
            buffer[3]='N';
            buffer[4]='S';
            buffer[5]='T';
            buffer[6]='R';
            buffer[7]='C';
            buffer[8]='A';
            buffer[9]='M';
            buffer[10]='#';
            buffer[11]='7';
            buffer[12]='8';
            buffer[13]='!';*/
//{
          //  while(1)
          //  {
            status = CyU3PI2cTransmitBytes (&preamble, buffer,byteCount, 0);
        //    }
//}
            if (status != CY_U3P_SUCCESS)
            {
                return status;
            }


            /* Wait for the write to complete. */
            preamble.length = 1;
            status = CyU3PI2cWaitForAck(&preamble, 200);
            if (status != CY_U3P_SUCCESS)
            {
                return status;
            }
        }

        /* An additional delay seems to be required after receiving an ACK. */
        CyU3PThreadSleep (1);

        /* Update the parameters */
      //  byteAddress  += glI2cPageSize;
       // buffer += glI2cPageSize;
       // pageCount --;
   // }

    return CY_U3P_SUCCESS;
}

/* This function starts the slave FIFO loop application. This is called
 * when a SET_CONF event is received from the USB host. The endpoints
 * are configured and the DMA pipe is setup in this function. */
void
CyFxSlFifoApplnStart (
        void)
{
    uint16_t size = 0;
    CyU3PEpConfig_t epCfg;
    CyU3PDmaChannelConfig_t dmaCfg;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PUSBSpeed_t usbSpeed = CyU3PUsbGetSpeed();

    /* First identify the usb speed. Once that is identified,
     * create a DMA channel and start the transfer on this. */

    /* Based on the Bus Speed configure the endpoint packet size */
    switch (usbSpeed)
    {
        case CY_U3P_FULL_SPEED:
            size = 64;
            break;

        case CY_U3P_HIGH_SPEED:
            size = 512;
            break;

        case  CY_U3P_SUPER_SPEED:
            size = 1024;
            break;

        default:
            CyU3PDebugPrint (4, "Error! Invalid USB speed.\n");
            CyFxAppErrorHandler (CY_U3P_ERROR_FAILURE);
            break;
    }

    CyU3PMemSet ((uint8_t *)&epCfg, 0, sizeof (epCfg));
    epCfg.enable = CyTrue;
    epCfg.epType = CY_U3P_USB_EP_BULK;
    epCfg.burstLen = 1;
    epCfg.streams = 0;
    epCfg.pcktSize = size;

    /* Producer endpoint configuration */
    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_PRODUCER, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PSetEpConfig failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler (apiRetStatus);
    }

    /* Consumer endpoint configuration */
    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONSUMER, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PSetEpConfig failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler (apiRetStatus);
    }

    /* Consumer2 endpoint configuration */
       apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONSUMER2, &epCfg);
       if (apiRetStatus != CY_U3P_SUCCESS)
       {
           CyU3PDebugPrint (4, "CyU3PSetEpConfig failed, Error code = %d\n", apiRetStatus);
         //  CyFxAppErrorHandler (apiRetStatus);
       }

       // commentd by geethanjali 10-04-2019//
    /* Create a DMA MANUAL channel for U2P transfer.
     * DMA size is set based on the USB speed. */
   // dmaCfg.size  = size;
   // dmaCfg.count = CY_FX_SLFIFO_DMA_BUF_COUNT;
 /*  dmaCfg.size  =32768;//8192;//512;
    dmaCfg.count = 2;//4;
    dmaCfg.prodSckId = CY_FX_PRODUCER_USB_SOCKET;
    dmaCfg.consSckId = CY_FX_CONSUMER_PPORT_SOCKET;
    dmaCfg.dmaMode = CY_U3P_DMA_MODE_BYTE;
    // Enabling the callback for produce event.
    dmaCfg.notification = CY_U3P_DMA_CB_PROD_EVENT;
    dmaCfg.cb = CyFxSlFifoUtoPDmaCallback;
    dmaCfg.prodHeader = 0;
    dmaCfg.prodFooter = 0;
    dmaCfg.consHeader = 0;
    dmaCfg.prodAvailCount = 0;

    apiRetStatus = CyU3PDmaChannelCreate (&glChHandleSlFifoUtoP,
            CY_U3P_DMA_TYPE_MANUAL, &dmaCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PDmaChannelCreate failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }*/


       // added by geethanjali 10-04-2019//
              //  dmaCfg.size  = size;
          // dmaCfg.count = CY_FX_SLFIFO_DMA_BUF_COUNT;
           dmaCfg.size  =32768;//8192;//512;
           dmaCfg.count = 2;//4;
           dmaCfg.prodSckId = CY_FX_PRODUCER_USB_SOCKET;
           dmaCfg.consSckId =CY_U3P_CPU_SOCKET_CONS ;
           dmaCfg.dmaMode = CY_U3P_DMA_MODE_BYTE;
           // Enabling the callback for produce event.
           dmaCfg.notification = CY_U3P_DMA_CB_PROD_EVENT;
           dmaCfg.cb =CyFxBulkSrcSinkDmaCallback1;
           dmaCfg.prodHeader = 0;
           dmaCfg.prodFooter = 0;
           dmaCfg.consHeader = 0;
           dmaCfg.prodAvailCount = 0;

           apiRetStatus = CyU3PDmaChannelCreate (&glChHandleSlFifoUtoP,
        		   CY_U3P_DMA_TYPE_MANUAL_IN, &dmaCfg);
           if (apiRetStatus != CY_U3P_SUCCESS)
           {
               CyU3PDebugPrint (4, "CyU3PDmaChannelCreate failed, Error code = %d\n", apiRetStatus);
               CyFxAppErrorHandler(apiRetStatus);
           }
//    /* Create a DMA MANUAL channel for P2U transfer. */
//    dmaCfg.size  = 32768;//512;//512;
//    dmaCfg.count = 2;//8;
//    dmaCfg.prodSckId = CY_FX_PRODUCER_PPORT_SOCKET;
//    dmaCfg.consSckId = CY_FX_CONSUMER_USB_SOCKET;
//
//  // dmaCfg.cb = NULL;
//  // dmaCfg.notification = 0;
//   dmaCfg.notification =CY_U3P_DMA_CB_PROD_EVENT;
//   dmaCfg.cb = CyFxSlFifoPtoUDmaCallback;
//    apiRetStatus = CyU3PDmaChannelCreate (&glChHandleSlFifoPtoU,
//    		CY_U3P_DMA_TYPE_AUTO, &dmaCfg);
//    if (apiRetStatus != CY_U3P_SUCCESS)
//    {
//        CyU3PDebugPrint (4, "CyU3PDmaChannelCreate failed, Error code = %d\n", apiRetStatus);
//        CyFxAppErrorHandler(apiRetStatus);
//    }

    /* Create a DMA MANUAL channel for P2S transfer. */
    dmaCfg.size  = 32768;//512;//512;
    dmaCfg.count = 2;//8;
    dmaCfg.prodSckId = CY_FX_PRODUCER_PPORT_SOCKET;
    dmaCfg.consSckId = CY_FX_CONSUMER_SPORT_SOCKET;// CY_U3P_SIB_SOCKET_0;

  // dmaCfg.cb = NULL;
  // dmaCfg.notification = 0;
    dmaCfg.notification = CY_U3P_DMA_CB_CONS_EVENT ;
    dmaCfg.cb = CyFxSlFifoPtoSDmaCallback;
    apiRetStatus = CyU3PDmaChannelCreate (&glChHandleSlFifoPtoS,
    		CY_U3P_DMA_TYPE_AUTO, &dmaCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PDmaChannelCreate failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

/* Create a DMA MANUAL_OUT channel for the consumer2 socket. */
	  dmaCfg.notification = CY_U3P_DMA_CB_CONS_EVENT;
	  dmaCfg.prodSckId = CY_U3P_CPU_SOCKET_PROD;    //
	  dmaCfg.consSckId = CY_FX_EP_CONSUMER_SOCKET;
	  dmaCfg.cb = CyFxBulkSrcSinkDmaCallback;;
	  apiRetStatus = CyU3PDmaChannelCreate (&glChHandleBulkSrc,
			  CY_U3P_DMA_TYPE_MANUAL_OUT, &dmaCfg);
	  if (apiRetStatus != CY_U3P_SUCCESS)
	  {
		  CyU3PDebugPrint (4, "CyU3PDmaChannelCreate failed, Error code = %d\n", apiRetStatus);
		 // CyFxAppErrorHandler(apiRetStatus);
	  }




    /* Flush the Endpoint memory */
    CyU3PUsbFlushEp(CY_FX_EP_PRODUCER);
    CyU3PUsbFlushEp(CY_FX_EP_CONSUMER);

    /* Set DMA channel transfer size. */
    apiRetStatus = CyU3PDmaChannelSetXfer (&glChHandleSlFifoUtoP, CY_FX_SLFIFO_DMA_TX_SIZE);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PDmaChannelSetXfer Failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }
    apiRetStatus = CyU3PDmaChannelSetXfer (&glChHandleSlFifoPtoS, CY_FX_SLFIFO_DMA_RX_SIZE);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PDmaChannelSetXfer Failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }
    apiRetStatus = CyU3PDmaChannelSetXfer (&glChHandleBulkSrc, CY_FX_SLFIFO_DMA_RX_SIZE);
          if (apiRetStatus != CY_U3P_SUCCESS)
          {
              CyU3PDebugPrint (4, "CyU3PDmaChannelSetXfer failed, Error code = %d\n", apiRetStatus);
          }
   /* Update the status flag. */
    glIsApplnActive = CyTrue;

}

/* This function stops the slave FIFO loop application. This shall be called
 * whenever a RESET or DISCONNECT event is received from the USB host. The
 * endpoints are disabled and the DMA pipe is destroyed by this function. */
void
CyFxSlFifoApplnStop (
        void)
{
    CyU3PEpConfig_t epCfg;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Update the flag. */
    glIsApplnActive = CyFalse;

    /* Flush the endpoint memory */
    CyU3PUsbFlushEp(CY_FX_EP_PRODUCER);
    CyU3PUsbFlushEp(CY_FX_EP_CONSUMER);

    /* Destroy the channel */
    CyU3PDmaChannelDestroy (&glChHandleSlFifoUtoP);
    CyU3PDmaChannelDestroy (&glChHandleSlFifoPtoS);

    /* Disable endpoints. */
    CyU3PMemSet ((uint8_t *)&epCfg, 0, sizeof (epCfg));
    epCfg.enable = CyFalse;

    /* Producer endpoint configuration. */
    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_PRODUCER, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PSetEpConfig failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler (apiRetStatus);
    }

    /* Consumer endpoint configuration. */
    apiRetStatus = CyU3PSetEpConfig(CY_FX_EP_CONSUMER, &epCfg);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PSetEpConfig failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler (apiRetStatus);
    }
}

/* Callback to handle the USB setup requests. */
CyBool_t
CyFxSlFifoApplnUSBSetupCB (
        uint32_t setupdat0,
        uint32_t setupdat1
    )
{
    /* Fast enumeration is used. Only requests addressed to the interface, class,
     * vendor and unknown control requests are received by this function.
     * This application does not support any class or vendor requests. */

    uint8_t  bRequest, bReqType;
    uint8_t  bType, bTarget;
    uint16_t wValue, wIndex;
    CyBool_t isHandled = CyFalse;

    /* Decode the fields from the setup request. */
    bReqType = (setupdat0 & CY_U3P_USB_REQUEST_TYPE_MASK);
    bType    = (bReqType & CY_U3P_USB_TYPE_MASK);
    bTarget  = (bReqType & CY_U3P_USB_TARGET_MASK);
    bRequest = ((setupdat0 & CY_U3P_USB_REQUEST_MASK) >> CY_U3P_USB_REQUEST_POS);
    wValue   = ((setupdat0 & CY_U3P_USB_VALUE_MASK)   >> CY_U3P_USB_VALUE_POS);
    wIndex   = ((setupdat1 & CY_U3P_USB_INDEX_MASK)   >> CY_U3P_USB_INDEX_POS);

    if (bType == CY_U3P_USB_STANDARD_RQT)
    {
        /* Handle SET_FEATURE(FUNCTION_SUSPEND) and CLEAR_FEATURE(FUNCTION_SUSPEND)
         * requests here. It should be allowed to pass if the device is in configured
         * state and failed otherwise. */
        if ((bTarget == CY_U3P_USB_TARGET_INTF) && ((bRequest == CY_U3P_USB_SC_SET_FEATURE)
                    || (bRequest == CY_U3P_USB_SC_CLEAR_FEATURE)) && (wValue == 0))
        {
            if (glIsApplnActive)
                CyU3PUsbAckSetup ();
            else
                CyU3PUsbStall (0, CyTrue, CyFalse);

            isHandled = CyTrue;
        }

        /* CLEAR_FEATURE request for endpoint is always passed to the setup callback
         * regardless of the enumeration model used. When a clear feature is received,
         * the previous transfer has to be flushed and cleaned up. This is done at the
         * protocol level. Since this is just a loopback operation, there is no higher
         * level protocol. So flush the EP memory and reset the DMA channel associated
         * with it. If there are more than one EP associated with the channel reset both
         * the EPs. The endpoint stall and toggle / sequence number is also expected to be
         * reset. Return CyFalse to make the library clear the stall and reset the endpoint
         * toggle. Or invoke the CyU3PUsbStall (ep, CyFalse, CyTrue) and return CyTrue.
         * Here we are clearing the stall. */
        if ((bTarget == CY_U3P_USB_TARGET_ENDPT) && (bRequest == CY_U3P_USB_SC_CLEAR_FEATURE)
                && (wValue == CY_U3P_USBX_FS_EP_HALT))
        {
            if (glIsApplnActive)
            {
                if (wIndex == CY_FX_EP_PRODUCER)
                {
                	 CyU3PUsbSetEpNak (CY_FX_EP_PRODUCER, CyTrue);
                	                  //  CyU3PBusyWait (125);
                    CyU3PDmaChannelReset (&glChHandleSlFifoUtoP);
                    CyU3PUsbFlushEp(CY_FX_EP_PRODUCER);
                    CyU3PUsbResetEp (CY_FX_EP_PRODUCER);
                    CyU3PUsbSetEpNak (CY_FX_EP_PRODUCER, CyFalse);
                    CyU3PDmaChannelSetXfer (&glChHandleSlFifoUtoP, CY_FX_SLFIFO_DMA_TX_SIZE);
                    CyU3PUsbStall (wIndex, CyFalse, CyTrue);

                                    CyU3PUsbAckSetup ();
                                    isHandled = CyTrue;
                }

                if (wIndex == CY_FX_EP_CONSUMER)
                {
                	 CyU3PUsbSetEpNak (CY_FX_EP_CONSUMER, CyTrue);
                	                 //   CyU3PBusyWait (125);
                    CyU3PDmaChannelReset (&glChHandleSlFifoPtoS);
                    CyU3PUsbFlushEp(CY_FX_EP_CONSUMER);
                   CyU3PUsbResetEp (CY_FX_EP_CONSUMER);
                   CyU3PUsbSetEpNak (CY_FX_EP_CONSUMER, CyFalse);
                    CyU3PDmaChannelSetXfer (&glChHandleSlFifoPtoS, CY_FX_SLFIFO_DMA_RX_SIZE);
                    CyU3PUsbStall (wIndex, CyFalse, CyTrue);

                                    CyU3PUsbAckSetup ();
                                    isHandled = CyTrue;
                }


            }
        }
    }

    return isHandled;
}

/* This is the callback function to handle the USB events. */
void
CyFxSlFifoApplnUSBEventCB (
    CyU3PUsbEventType_t evtype,
    uint16_t            evdata
    )
{
    switch (evtype)
    {
        case CY_U3P_USB_EVENT_SETCONF:
            /* Stop the application before re-starting. */
            if (glIsApplnActive)
            {
                CyFxSlFifoApplnStop ();
            }
            /* Start the loop back function. */
            CyFxSlFifoApplnStart ();
            break;

        case CY_U3P_USB_EVENT_RESET:
        case CY_U3P_USB_EVENT_DISCONNECT:
            /* Stop the loop back function. */
            if (glIsApplnActive)
            {
                CyFxSlFifoApplnStop ();
            }
            break;

        default:
            break;
    }
}



/* Callback function to handle LPM requests from the USB 3.0 host. This function is invoked by the API
   whenever a state change from U0 -> U1 or U0 -> U2 happens. If we return CyTrue from this function, the
   FX3 device is retained in the low power state. If we return CyFalse, the FX3 device immediately tries
   to trigger an exit back to U0.

   This application does not have any state in which we should not allow U1/U2 transitions; and therefore
   the function always return CyTrue.
 */
CyBool_t
CyFxApplnLPMRqtCB (
        CyU3PUsbLinkPowerMode link_mode)
{
    return CyTrue;
}

void
CyFxGpifAppSibCallback (
        uint8_t             portId,
        CyU3PSibEventType   evt,
        CyU3PReturnStatus_t status)
{
   // CyU3PMbox evtMbox;
	switch(evt)
	    {

	        case CY_U3P_SIB_EVENT_INSERT:

	        if (glIsApplnActive)
	            	            {
	        					CyFxSlFifoApplnStart();
	            	            }
	            	            CyU3PUsbLPMDisable();
	            	            CyFxSlFifoApplnStart ();
	        break;

	        case CY_U3P_SIB_EVENT_REMOVE:
	        if (glIsApplnActive)
	            	            {
	            	                CyFxSlFifoApplnStop ();
	            	            }
	        break;


	    }

}


/* This function initializes the GPIF interface and initializes
 * the USB interface. */
void
CyFxSlFifoApplnInit (void)
{
    CyU3PPibClock_t pibClock;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;

    /* Initialize the p-port block. */
    pibClock.clkDiv =16;
    pibClock.clkSrc = CY_U3P_SYS_CLK;
    pibClock.isHalfDiv = CyFalse;
    /* Disable DLL for sync GPIF */
    pibClock.isDllEnable = CyFalse;
    apiRetStatus = CyU3PPibInit(CyTrue, &pibClock);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "P-port Initialization failed, Error Code = %d\n",apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Load the GPIF configuration for Slave FIFO sync mode. */
 //   apiRetStatus = CyU3PGpifLoad (&Sync_Slave_Fifo_2Bit_CyFxGpifConfig);//changed by veerendra
   apiRetStatus = CyU3PGpifLoad (&CyFxGpifConfig);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PGpifLoad failed, Error Code = %d\n",apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }
    CyU3PGpifSocketConfigure (0,CY_U3P_PIB_SOCKET_0,3,CyFalse,1);
    CyU3PGpifSocketConfigure (3,CY_U3P_PIB_SOCKET_3,6,CyFalse,1);
    /* Start the state machine. */
   // apiRetStatus = CyU3PGpifSMStart (SYNC_SLAVE_FIFO_2BIT_RESET, SYNC_SLAVE_FIFO_2BIT_ALPHA_RESET);  //changed by veerendra
    apiRetStatus = CyU3PGpifSMStart (RESET, ALPHA_RESET);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PGpifSMStart failed, Error Code = %d\n",apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }
    //CyU3PPibRegisterCallback(gpif_error_cb,0xffff);


    /* Start the USB functionality. */
        apiRetStatus= CyU3PUsbStart();
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "CyU3PUsbStart failed to Start, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

  // CyU3PPibRegisterCallback(gpif_error_cb,0xffff);
    /* The fast enumeration is the easiest way to setup a USB connection,
     * where all enumeration phase is handled by the library. Only the
     * class / vendor requests need to be handled by the application. */
    CyU3PUsbRegisterSetupCallback(CyFxSlFifoApplnUSBSetupCB, CyTrue);

    /* Setup the callback to handle the USB events. */
    CyU3PUsbRegisterEventCallback(CyFxSlFifoApplnUSBEventCB);

    /* Register a callback to handle LPM requests from the USB 3.0 host. */
    CyU3PUsbRegisterLPMRequestCallback(CyFxApplnLPMRqtCB);    

    /* Set the USB Enumeration descriptors */

    /* Super speed device descriptor. */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_DEVICE_DESCR, 0, (uint8_t *)CyFxUSB30DeviceDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set device descriptor failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* High speed device descriptor. */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_HS_DEVICE_DESCR, 0, (uint8_t *)CyFxUSB20DeviceDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set device descriptor failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* BOS descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_BOS_DESCR, 0, (uint8_t *)CyFxUSBBOSDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set configuration descriptor failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Device qualifier descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_DEVQUAL_DESCR, 0, (uint8_t *)CyFxUSBDeviceQualDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set device qualifier descriptor failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Super speed configuration descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_SS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBSSConfigDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set configuration descriptor failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* High speed configuration descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_HS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBHSConfigDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB Set Other Speed Descriptor failed, Error Code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Full speed configuration descriptor */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_FS_CONFIG_DESCR, 0, (uint8_t *)CyFxUSBFSConfigDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB Set Configuration Descriptor failed, Error Code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* String descriptor 0 */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 0, (uint8_t *)CyFxUSBStringLangIDDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set string descriptor failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* String descriptor 1 */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 1, (uint8_t *)CyFxUSBManufactureDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set string descriptor failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* String descriptor 2 */
    apiRetStatus = CyU3PUsbSetDesc(CY_U3P_USB_SET_STRING_DESCR, 2, (uint8_t *)CyFxUSBProductDscr);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB set string descriptor failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }

    /* Connect the USB Pins with super speed operation enabled. */
    apiRetStatus = CyU3PConnectState(CyTrue, CyTrue);
    if (apiRetStatus != CY_U3P_SUCCESS)
    {
        CyU3PDebugPrint (4, "USB Connect failed, Error code = %d\n", apiRetStatus);
        CyFxAppErrorHandler(apiRetStatus);
    }
}


/* Entry function for the slFifoAppThread. */
void
SlFifoAppThread_Entry (
        uint32_t input)
{
    /* Initialize the debug module */
    CyFxSlFifoApplnDebugInit();

    /* Initialize the slave FIFO application */
    CyFxSlFifoApplnInit();

    for (;;)
    {
        CyU3PThreadSleep (1000);
       if (glIsApplnActive)
       {
          if (flag==0x55)
           {
        	  	 xyz();
              	 flag =0x00;
            }
   	    }
     }
    if(get_flag)
    {
    	CyFxBulkSrcSinkFillInBuffers();
    }
 }
void
AppThread_Entry1 (
        uint32_t input)
{
   // uint8_t count = 0;
    CyU3PReturnStatus_t apiRetStatus = CY_U3P_SUCCESS;
    CyU3PI2cConfig_t i2cConfig;
           apiRetStatus = CyU3PI2cInit ();
                   if (apiRetStatus != CY_U3P_SUCCESS)
                   {
                   	CyU3PDebugPrint (4, "i2c init failed to Start, Error code = %d\n", apiRetStatus);
                   	        	        CyFxAppErrorHandler(apiRetStatus);
                   }

                   /* Start the I2C master block. The bit rate is set at 100KHz.
                    * The data transfer is done via DMA. */
                   CyU3PMemSet ((uint8_t *)&i2cConfig, 0, sizeof(i2cConfig));
                   i2cConfig.bitRate    = CY_FX_USBI2C_I2C_BITRATE;
                   i2cConfig.busTimeout = 0xFFFFFFFF;
                   i2cConfig.dmaTimeout = 0xFFFF;
                   i2cConfig.isDma      = CyFalse;

                   apiRetStatus  = CyU3PI2cSetConfig (&i2cConfig, 0);

                   if (apiRetStatus != CY_U3P_SUCCESS)
                   {
                   	 CyU3PDebugPrint (4, "i2c configuration failed to Start, Error code = %d\n", apiRetStatus);
                   	        CyFxAppErrorHandler(apiRetStatus);
                   }






    if (  apiRetStatus != CY_U3P_SUCCESS)
    {
        goto handle_error;
    }

    for (;;)
    {
        //CyU3PDebugPrint (4, "%x: Device initialized. Firmware ID: %x %x %x %x %x %x %x %x\r\n",
              //  count++, glFirmwareID[3], glFirmwareID[2], glFirmwareID[1], glFirmwareID[0],
              //  glFirmwareID[7], glFirmwareID[6], glFirmwareID[5], glFirmwareID[4]);
        CyU3PThreadSleep (1000);
    }

handle_error:
    CyU3PDebugPrint (4, "%x: Application failed to initialize. Error code: %d.\n", apiRetStatus);
    while (1);
}



/* Application define function which creates the threads. */
void
CyFxApplicationDefine (
        void)
{
    void *ptr = 0;
    uint32_t retThrdCreate = CY_U3P_SUCCESS;

    /* Allocate the memory for the thread */
    ptr = CyU3PMemAlloc (CY_FX_SLFIFO_THREAD_STACK);

    /* Create the thread for the application */
    retThrdCreate = CyU3PThreadCreate (&slFifoAppThread,           /* Slave FIFO app thread structure */
                          "21:Slave_FIFO_sync",                    /* Thread ID and thread name */
                          SlFifoAppThread_Entry,                   /* Slave FIFO app thread entry function */
                          0,                                       /* No input parameter to thread */
                          ptr,                                     /* Pointer to the allocated thread stack */
                          CY_FX_SLFIFO_THREAD_STACK,               /* App Thread stack size */
                          CY_FX_SLFIFO_THREAD_PRIORITY,            /* App Thread priority */
                          CY_FX_SLFIFO_THREAD_PRIORITY,            /* App Thread pre-emption threshold */
                          CYU3P_NO_TIME_SLICE,                     /* No time slice for the application thread */
                          CYU3P_AUTO_START                         /* Start the thread immediately */
                          );

    ptr = CyU3PMemAlloc (APPTHREAD_STACK);
        retThrdCreate = CyU3PThreadCreate (&appThread, /* Thread structure. */
                "21:AppThread",                        /* Thread ID and name. */
                AppThread_Entry1,                       /* Thread entry function. */
                0,                                     /* Thread input parameter. */
                ptr,                                   /* Pointer to the allocated thread stack. */
                APPTHREAD_STACK,                       /* Allocated thread stack size. */
                APPTHREAD_PRIORITY,                    /* Thread priority. */
                APPTHREAD_PRIORITY,                    /* Thread pre-emption threshold: No preemption. */
                CYU3P_NO_TIME_SLICE,                   /* No time slice. Thread will run until task is
                                                          completed or until the higher priority
                                                          thread gets active. */
                CYU3P_AUTO_START                       /* Start the thread immediately. */
                );

    /* Check the return code */
    if (retThrdCreate != 0)
    {
        /* Thread Creation failed with the error code retThrdCreate */

        /* Add custom recovery or debug actions here */

        /* Application cannot continue */
        /* Loop indefinitely */
        while(1);
    }
}
void xyz()
{


if(recieved_from_usb==1 && ZYNQ_COMM==1 && checksunm_flag==1)
{
	 CyFxUsbI2cTransfer (50,receivecom,CyFalse);
	 ZYNQ_COMM=0;
	 recieved_from_usb=0;
	 checksunm_flag=0;
}
	 // CyU3PThreadSleep (1000);
	  //CyFxUsbI2cTransfer (CyTrue);
	 // CyFxBulkSrcSinkFillInBuffers ();


}
/*
 * Main function
 */
int
main (void)
{
    CyU3PIoMatrixConfig_t io_cfg;
    CyU3PReturnStatus_t status = CY_U3P_SUCCESS;
    CyU3PSysClockConfig_t clockConfig;

    /* When the GPIF data bus is configured as 32-bits wide and running at 100 MHz (synchronous),
       the FX3 system clock has to be set to a frequency greater than 400 MHz. */
#if (CY_FX_SLFIFO_GPIF_16_32BIT_CONF_SELECT == 0)
    clockConfig.setSysClk400  = CyFalse;
#else
    clockConfig.setSysClk400  = CyTrue;
#endif
    clockConfig.cpuClkDiv     = 2;
    clockConfig.dmaClkDiv     = 2;
    clockConfig.mmioClkDiv    = 2;
    clockConfig.useStandbyClk = CyFalse;
    clockConfig.clkSrc        = CY_U3P_SYS_CLK;
    status = CyU3PDeviceInit (&clockConfig);
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* Initialize the caches. Enable both Instruction and Data Caches. */
    status = CyU3PDeviceCacheControl (CyTrue, CyTrue, CyTrue);
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* Configure the IO matrix for the device. On the FX3 DVK board, the COM port 
     * is connected to the IO(53:56). This means that either DQ32 mode should be
     * selected or lppMode should be set to UART_ONLY. Here we are choosing
     * UART_ONLY configuration for 16 bit slave FIFO configuration and setting
     * isDQ32Bit for 32-bit slave FIFO configuration. */
    io_cfg.s0Mode = CY_U3P_SPORT_INACTIVE;
    io_cfg.s1Mode = CY_U3P_SPORT_INACTIVE;
    io_cfg.useUart   = CyTrue;
    io_cfg.useI2C    = CyTrue;
    io_cfg.useI2S    = CyFalse;
    io_cfg.useSpi    = CyFalse;
#if (CY_FX_SLFIFO_GPIF_16_32BIT_CONF_SELECT == 0)
    io_cfg.isDQ32Bit = CyFalse;
    io_cfg.lppMode   = CY_U3P_IO_MATRIX_LPP_DEFAULT;
#else
    io_cfg.isDQ32Bit = CyTrue;
    io_cfg.lppMode   = CY_U3P_IO_MATRIX_LPP_DEFAULT;
#endif
    /* No GPIOs are enabled. */
    io_cfg.gpioSimpleEn[0]  = 0;
    io_cfg.gpioSimpleEn[1]  = 0x01E00000; //for spi
    io_cfg.gpioComplexEn[0] = 0;
    io_cfg.gpioComplexEn[1] = 0;
    status = CyU3PDeviceConfigureIOMatrix (&io_cfg);
    if (status != CY_U3P_SUCCESS)
    {
        goto handle_fatal_error;
    }

    /* This is a non returnable call for initializing the RTOS kernel */
    CyU3PKernelEntry ();

    /* Dummy return to make the compiler happy */
    return 0;

handle_fatal_error:

    /* Cannot recover from this error. */
    while (1);
}

/* [ ] */
//pib callback

void
gpif_error_cb(CyU3PPibIntrType cbType, uint16_t cbArg)
{

if(cbType==CYU3P_PIB_INTR_ERROR)
{
    switch (CYU3P_GET_PIB_ERROR_TYPE(cbArg))
    {
        case CYU3P_PIB_ERR_THR0_WR_OVERRUN:
        CyU3PDebugPrint (4, "CYU3P_PIB_ERR_THR0_WR_OVERRUN");
        break;
        case CYU3P_PIB_ERR_THR1_WR_OVERRUN:
        CyU3PDebugPrint (4, "CYU3P_PIB_ERR_THR1_WR_OVERRUN");
        break;
        case CYU3P_PIB_ERR_THR2_WR_OVERRUN:
        CyU3PDebugPrint (4, "CYU3P_PIB_ERR_THR2_WR_OVERRUN");
        break;
        case CYU3P_PIB_ERR_THR3_WR_OVERRUN:
        CyU3PDebugPrint (4, "CYU3P_PIB_ERR_THR3_WR_OVERRUN");
        break;

        case CYU3P_PIB_ERR_THR0_RD_UNDERRUN:
        CyU3PDebugPrint (4, "CYU3P_PIB_ERR_THR0_RD_UNDERRUN");
        break;
        case CYU3P_PIB_ERR_THR1_RD_UNDERRUN:
        CyU3PDebugPrint (4, "CYU3P_PIB_ERR_THR1_RD_UNDERRUN");
        break;
        case CYU3P_PIB_ERR_THR2_RD_UNDERRUN:
        CyU3PDebugPrint (4, "CYU3P_PIB_ERR_THR2_RD_UNDERRUN");
        break;
        case CYU3P_PIB_ERR_THR3_RD_UNDERRUN:
        CyU3PDebugPrint (4, "CYU3P_PIB_ERR_THR3_RD_UNDERRUN");
        break;

        default:
        CyU3PDebugPrint (4, "No Error :%d\n ",CYU3P_GET_PIB_ERROR_TYPE(cbArg));
            break;
    }
}
}

